@@ -1,6 +1,6 @@
 /* Copyright (C) 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: listcontrol.c,v 1.30 2004/11/09 15:19:04 mmj Exp $
+ * $Id: listcontrol.c,v 1.35 2005/02/03 14:53:15 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -41,6 +41,7 @@
 #include "mygetline.h"
 #include "chomp.h"
 #include "memory.h"
+#include "log_oper.h"
 
 enum ctrl_e {
 	CTRL_SUBSCRIBE_DIGEST,
@@ -62,8 +63,6 @@ enum ctrl_e {
 	CTRL_END  /* end marker, must be last */
 };
 
-/* MMJ says that function pointers are just for show off, so I will
- * switch an enum :-)  -- mortenp 20040511 */
 struct ctrl_command {
 	char *command;
 	unsigned int accepts_parameter;
@@ -103,7 +102,7 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 	const char *subswitch;
 	size_t len;
 	struct stat stbuf;
-	int closedlist, nosubconfirm, tmpfd;
+	int closedlist, nosubconfirm, tmpfd, noget;
 	size_t cmdlen;
 	unsigned int ctrl;
 	
@@ -172,11 +171,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		if (!strchr(fromemails->emaillist[0], '@'))
 			/* Not a valid From: address, silently ignore */
 			exit(EXIT_SUCCESS);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for digest"
+					" subscription from %s",
+					fromemails->emaillist[0]);
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
 				"-d",
-				subswitch, NULL);
+				subswitch, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -190,11 +192,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		if (!strchr(fromemails->emaillist[0], '@'))
 			/* Not a valid From: address, silently ignore */
 			exit(EXIT_SUCCESS);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for nomail"
+					" subscription from %s",
+					fromemails->emaillist[0]);
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
 				"-n",
-				subswitch, NULL);
+				subswitch, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -208,10 +213,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		if (!strchr(fromemails->emaillist[0], '@'))
 			/* Not a valid From: address, silently ignore */
 			exit(EXIT_SUCCESS);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for regular"
+					" subscription from %s",
+					fromemails->emaillist[0]);
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
-				subswitch, NULL);
+				subswitch, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -232,11 +240,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		chomp(tmpstr);
 		close(tmpfd);
 		unlink(conffilename);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: %s confirmed"
+					" subscription to digest", tmpstr);
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-d",
-				"-c", NULL);
+				"-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -257,11 +267,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		chomp(tmpstr);
 		close(tmpfd);
 		unlink(conffilename);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: %s confirmed"
+					" subscription to nomail", tmpstr);
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-n",
-				"-c", NULL);
+				"-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -282,10 +294,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		chomp(tmpstr);
 		close(tmpfd);
 		unlink(conffilename);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: %s confirmed"
+					" subscription to regular list",
+					tmpstr);
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", tmpstr,
-				"-c", NULL);
+				"-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -299,11 +314,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		if (!strchr(fromemails->emaillist[0], '@'))
 			/* Not a valid From: address, silently ignore */
 			exit(EXIT_SUCCESS);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
+					" unsubscribe from digest",
+					fromemails->emaillist[0]);
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
 				"-d",
-				subswitch, NULL);
+				subswitch, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -317,11 +335,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		if (!strchr(fromemails->emaillist[0], '@'))
 			/* Not a valid From: address, silently ignore */
 			exit(EXIT_SUCCESS);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
+					" unsubscribe from nomail",
+					fromemails->emaillist[0]);
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
 				"-n",
-				subswitch, NULL);
+				subswitch, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -335,10 +356,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		if (!strchr(fromemails->emaillist[0], '@'))
 			/* Not a valid From: address, silently ignore */
 			exit(EXIT_SUCCESS);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
+					" unsubscribe from regular list",
+					fromemails->emaillist[0]);
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
-				subswitch, NULL);
+				subswitch, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -359,11 +383,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		close(tmpfd);
 		chomp(tmpstr);
 		unlink(conffilename);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s confirmed"
+					" unsubscribe from digest", tmpstr);
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-d",
-				"-c", NULL);
+				"-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -384,11 +410,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		close(tmpfd);
 		chomp(tmpstr);
 		unlink(conffilename);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s confirmed"
+					" unsubscribe from nomail", tmpstr);
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-n",
-				"-c", NULL);
+				"-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -409,28 +437,38 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		close(tmpfd);
 		chomp(tmpstr);
 		unlink(conffilename);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s confirmed"
+					" unsubscribe from regular list",
+					tmpstr);
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", tmpstr,
-				"-c", NULL);
+				"-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
 		break;
 
-	/* listname+bounces-user=example.tld-INDEX@domain.tld */
+	/* listname+bounces-INDEX-user=example.tld@domain.tld */
 	case CTRL_BOUNCES:
-		bouncenr = strrchr(param, '-');
-		if (!bouncenr) { /* malformed bounce, ignore and clean up */
-			unlink(mailname);
-			exit(EXIT_SUCCESS);
+		bouncenr = param;
+		c = strchr(param, '-');
+		if (!c) { /* Exec with dsn parsing, since the addr is missing */
+			execlp(mlmmjbounce, mlmmjbounce,
+					"-L", listdir,
+					"-m", mailname,
+					"-n", bouncenr,
+					"-d", (char *)NULL);
+			log_error(LOG_ARGS, "execlp() of '%s' failed",
+					mlmmjbounce);
+			exit(EXIT_FAILURE);
 		}
-		*bouncenr++ = '\0';
+		*c++ = '\0';
 		execlp(mlmmjbounce, mlmmjbounce,
 				"-L", listdir,
-				"-a", param,
+				"-a", c,
 				"-m", mailname,
-				"-n", bouncenr, NULL);
+				"-n", bouncenr, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjbounce);
 		exit(EXIT_FAILURE);
 		break;
@@ -445,9 +483,11 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 			myfree(moderatefilename);
 			exit(EXIT_SUCCESS); /* just exit, no mail to moderate */
 		}
+		log_oper(listdir, OPLOGFNAME, "%s moderated %s",
+				fromemails->emaillist[0], moderatefilename);
 		execlp(mlmmjsend, mlmmjsend,
 				"-L", listdir,
-				"-m", moderatefilename, NULL);
+				"-m", moderatefilename, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsend);
 		exit(EXIT_FAILURE);
@@ -456,14 +496,20 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 	/* listname+help@domain.tld */
 	case CTRL_HELP:
 		unlink(mailname);
-		if(strchr(fromemails->emaillist[0], '@'))
+		if(strchr(fromemails->emaillist[0], '@')) {
+			log_oper(listdir, OPLOGFNAME, "%s requested help",
+				fromemails->emaillist[0]);
 			send_help(listdir, fromemails->emaillist[0],
 				  mlmmjsend);
+		}
 		break;
 
 	/* listname+get-INDEX@domain.tld */
 	case CTRL_GET:
 		unlink(mailname);
+		noget = statctrl(listdir, "noget");
+		if(noget)
+			exit(EXIT_SUCCESS);
 		/* sanity check--is it all digits? */
 		for(c = param; *c != '\0'; c++) {
 			if(!isdigit((int)*c))
@@ -473,12 +519,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 						param);
 		if(stat(archivefilename, &stbuf) < 0)
 			exit(EXIT_SUCCESS);
+		log_oper(listdir, OPLOGFNAME, "%s got archive/%s",
+				fromemails->emaillist[0], archivefilename);
 		execlp(mlmmjsend, mlmmjsend,
 				"-T", fromemails->emaillist[0],
 				"-L", listdir,
 				"-l", "6",
 				"-m", archivefilename,
-				"-a", "-D", NULL);
+				"-a", "-D", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsend);
 		exit(EXIT_FAILURE);
