@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Morten K. Poulsen <morten at afdelingp.dk>
  *
- * $Id: mlmmj-bounce.c,v 1.29 2004/11/11 14:54:55 mmj Exp $
+ * $Id: mlmmj-bounce.c,v 1.35 2005/02/03 16:20:32 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -32,6 +32,8 @@
 #include <time.h>
 #include <dirent.h>
 #include <sys/mman.h>
+#include <string.h>
+#include <ctype.h>
 
 #include "getlistaddr.h"
 #include "mlmmj.h"
@@ -43,6 +45,8 @@
 #include "chomp.h"
 #include "prepstdreply.h"
 #include "memory.h"
+#include "find_email_adr.h"
+#include "gethdrline.h"
 
 char *fetchindexes(const char *bouncefile)
 {
@@ -108,7 +112,7 @@ void do_probe(const char *listdir, const char *mlmmjsend, const char *addr)
 	listname = genlistname(listaddr);
 	listfqdn = genlistfqdn(listaddr);
 
-	from = concatstr(5, listname, "+bounces-", myaddr, "-probe@", listfqdn);
+	from = concatstr(5, listname, "+bounces-probe-", myaddr, "@", listfqdn);
 
 	myfree(listaddr);
 	myfree(listfqdn);
@@ -157,13 +161,56 @@ void do_probe(const char *listdir, const char *mlmmjsend, const char *addr)
 				"-L", listdir,
 				"-T", myaddr,
 				"-F", from,
-				"-m", queuefilename, NULL);
+				"-m", queuefilename, (char *)NULL);
 
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
 
 	exit(EXIT_FAILURE);
 }
 
+char *dsnparseaddr(const char *mailname)
+{
+	int fd, indsn = 0, i;
+	char *line, *linedup, *search, *addr = NULL;
+	struct email_container emails = { 0, NULL };
+
+	fd = open(mailname, O_RDONLY);
+	if(fd < 0) {
+		log_error(LOG_ARGS, "Could not open bounceindexfile %s",
+				mailname);
+		return NULL;
+	}
+
+	while((line = gethdrline(fd))) {
+		linedup = mystrdup(line);
+		for(i = 0; line[i]; i++)
+			linedup[i] = tolower(line[i]);
+		search = strstr(linedup, "message/delivery-status");
+		myfree(linedup);
+		if(search)
+			indsn = 1;
+		if(indsn) {
+			i = strncasecmp(line, "Final-Recipient:", 16);
+			if(i == 0) {
+				find_email_adr(line, &emails);
+				if(emails.emailcount > 0) {
+					addr = mystrdup(emails.emaillist[0]);
+					for(i = 0; i < emails.emailcount; i++)
+						myfree(emails.emaillist[i]);
+					myfree(emails.emaillist);	
+				} else {
+					addr = NULL;
+				}
+				myfree(line);
+				return addr;
+			}
+		}
+		myfree(line);
+	}
+	
+	return NULL;
+}
+
 static void print_help(const char *prg)
 {
 	printf("Usage: %s -L /path/to/list -a john=doe.org [-n num | -p]\n"
@@ -179,7 +226,7 @@ static void print_help(const char *prg)
 
 int main(int argc, char **argv)
 {
-	int opt, fd;
+	int opt, fd, dsnbounce = 0;
 	char *listdir = NULL, *address = NULL, *number = NULL;
 	char *bindir, *mlmmjsend, *savename;
 	char *mailname = NULL, *bfilename, *a, *buf;
@@ -197,7 +244,7 @@ int main(int argc, char **argv)
 	mlmmjsend = concatstr(2, bindir, "/mlmmj-send");
 	myfree(bindir);
 
-	while ((opt = getopt(argc, argv, "hVL:a:n:m:p")) != -1) {
+	while ((opt = getopt(argc, argv, "hdVL:a:n:m:p")) != -1) {
 		switch(opt) {
 		case 'L':
 			listdir = optarg;
@@ -205,6 +252,9 @@ int main(int argc, char **argv)
 		case 'a':
 			address = optarg;
 			break;
+		case 'd':
+			dsnbounce = 1;
+			break;
 		case 'm':
 			mailname = optarg;
 			break;
@@ -223,8 +273,10 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if(listdir == NULL || address == NULL || (number == NULL && probe == 0)) {
-		fprintf(stderr, "You have to specify -L, -a and -n or -p\n");
+	if(listdir == NULL || (address == NULL && dsnbounce == 0)
+				|| (number == NULL && probe == 0)) {
+		fprintf(stderr,
+			"You have to specify -L, -a or -d and -n or -p\n");
 		fprintf(stderr, "%s -h for help\n", argv[0]);
 		exit(EXIT_FAILURE);
 	}
@@ -247,7 +299,19 @@ int main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
-	
+
+	if(dsnbounce) {
+		address = dsnparseaddr(mailname);
+		if(address == NULL) {
+			/* not parseable, so unlink and clean up */
+			if(mailname)
+				unlink(mailname);
+			exit(EXIT_SUCCESS);
+		}
+		a = strrchr(address, '@');
+		*a = '=';
+	}
+			
 	if(number != NULL && probe != 0) {
 		fprintf(stderr, "You can only specify one of -n or -p\n");
 		fprintf(stderr, "%s -h for help\n", argv[0]);
@@ -262,7 +326,7 @@ int main(int argc, char **argv)
 	}
 
 #if 0
-	log_error(LOG_ARGS, "[%s] [%s] [%s]", listdir, address, number);
+	log_error(LOG_ARGS, "listdir = [%s] address = [%s] number = [%s]", listdir, address, number);
 #endif
 
 	/* check if it's sub/unsub requests bouncing, and in that case
@@ -270,24 +334,23 @@ int main(int argc, char **argv)
 	 * number are a bit misleading in this case due to the different
 	 * construction of the sub/unsub confirmation From header.
 	 */
-	if(strncmp(address, "confsub-", 8) == 0) {
-		a = concatstr(5, listdir, "/subconf/", address + 8, "-",
-				number);
+	if(strcmp(number, "confsub") == 0) {
+		a = concatstr(3, listdir, "/subconf/", address);
 		unlink(a);
 		myfree(a);
 		exit(EXIT_SUCCESS);
 	}
-	if(strncmp(address, "confunsub-", 10) == 0) {
-		a = concatstr(5, listdir, "/unsubconf/", address + 10, "-",
-				number);
+	if(strcmp(number, "confunsub") == 0) {
+		a = concatstr(3, listdir, "/unsubconf/", address);
 		unlink(a);
 		myfree(a);
 		exit(EXIT_SUCCESS);
 	}
 	/* Below checks for bounce probes bouncing. If they do, simply remove
-	 * the probe file and exit successfully
+	 * the probe file and exit successfully. Yes, I know the variables
+	 * have horrible names, but please bear with me.
 	 */
-	if(strncmp(number, "probe", 5) == 0) {
+	if(strcmp(number, "probe") == 0) {
 		a = concatstr(4, listdir, "/bounce/", address, "-probe");
 		unlink(a);
 		unlink(mailname);
@@ -349,6 +412,8 @@ int main(int argc, char **argv)
 	}
 		
 	myfree(bfilename);
+	if(dsnbounce && address)
+		myfree(address);
 
 	return EXIT_SUCCESS;
 }
