@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-maintd.c,v 1.42 2004/11/12 01:28:32 mmj Exp $
+ * $Id: mlmmj-maintd.c,v 1.51 2005/02/14 09:32:02 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -44,8 +44,7 @@
 #include "ctrlvalue.h"
 #include "send_digest.h"
 #include "mylocking.h"
-
-static int maintdlogfd = -1;
+#include "log_oper.h"
 
 static void print_help(const char *prg)
 {
@@ -178,16 +177,29 @@ int discardmail(const char *old, const char *new, time_t age)
 {
 	struct stat st;
 	time_t t;
+	int fd, ret = 0;
+
+	fd = open(old, O_RDWR);
+	if(fd < 0)
+		return 0;
+
+	if(myexcllock(fd) < 0) {
+		close(fd);
+		return 0;
+	}
 
 	stat(old, &st);
 	t = time(NULL);
 
 	if(t - st.st_mtime > age) {
-		rename(old, new);
-		return 1;
+		if(rename(old, new) < 0)
+			ret = 0;
+		else
+			ret = 1;
 	}
 
-	return 0;
+	close(fd);
+	return ret;
 }
 
 int resend_queue(const char *listdir, const char *mlmmjsend)
@@ -195,8 +207,7 @@ int resend_queue(const char *listdir, const char *mlmmjsend)
 	DIR *queuedir;
 	struct dirent *dp;
 	char *mailname, *fromname, *toname, *reptoname, *from, *to, *repto;
-	char *discardedname = NULL, *ch;
-	char *dirname = concatstr(2, listdir, "/queue/");
+	char *ch, *dirname = concatstr(2, listdir, "/queue/");
 	pid_t childpid, pid;
 	struct stat st;
 	int fromfd, tofd, fd, discarded = 0, status;
@@ -237,38 +248,7 @@ int resend_queue(const char *listdir, const char *mlmmjsend)
 		mailname = concatstr(3, listdir, "/queue/", dp->d_name);
 
 		fromname = concatstr(2, mailname, ".mailfrom");
-		if(stat(fromname, &st) < 0) {
-			if(errno == ENOENT) {
-				discardedname = concatstr(4,
-						listdir, "/queue/discarded/",
-						dp->d_name, ".by-maintd");
-				discarded = discardmail(mailname,
-							discardedname,
-							3600);
-				myfree(discardedname);
-			} else {
-				log_error(LOG_ARGS, "Could not stat(%s)",
-						dp->d_name);
-			}
-		}
-		
-
 		toname = concatstr(2, mailname, ".reciptto");
-		if(!discarded && stat(toname, &st) < 0) {
-			if(errno == ENOENT) {
-				discardedname = concatstr(4,
-						listdir, "/queue/discarded/",
-						dp->d_name, ".by-maintd");
-				discarded = discardmail(mailname,
-							discardedname,
-							3600);
-				myfree(discardedname);
-			} else {
-				log_error(LOG_ARGS, "Could not stat(%s)",
-						dp->d_name);
-			}
-		}
-
 		reptoname = concatstr(2, mailname, ".reply-to");
 
 		fromfd = open(fromname, O_RDONLY);
@@ -339,14 +319,14 @@ int resend_queue(const char *listdir, const char *mlmmjsend)
 						"-F", from,
 						"-T", to,
 						"-R", repto,
-						"-a", NULL);
+						"-a", (char *)NULL);
 			} else {
 				execlp(mlmmjsend, mlmmjsend,
 						"-l", "1",
 						"-m", mailname,
 						"-F", from,
 						"-T", to,
-						"-a", NULL);
+						"-a", (char *)NULL);
 			}
 		}
 	}
@@ -410,7 +390,6 @@ int resend_requeue(const char *listdir, const char *mlmmjsend)
 		}
 		subfilename = concatstr(3, dirname, dp->d_name, "/subscribers");
 		if(stat(subfilename, &st) < 0) {
-			log_error(LOG_ARGS, "Could not stat(%s)", subfilename);
 			myfree(archivefilename);
 			myfree(subfilename);
 			continue;
@@ -451,7 +430,7 @@ int resend_requeue(const char *listdir, const char *mlmmjsend)
 					"-m", archivefilename,
 					"-s", subnewname,
 					"-a",
-					"-D", NULL);
+					"-D", (char *)NULL);
 		}
 	}
 
@@ -593,7 +572,7 @@ int probe_bouncers(const char *listdir, const char *mlmmjbounce)
 			execlp(mlmmjbounce, mlmmjbounce,
 					"-L", listdir,
 					"-a", probefile,
-					"-p", NULL);
+					"-p", (char *)NULL);
 			log_error(LOG_ARGS, "Could not execlp %s",
 						mlmmjbounce);
 			return 1;
@@ -609,7 +588,7 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 	DIR *bouncedir;
 	char *dirname = concatstr(2, listdir, "/bounce/");
 	char *probefile, *address, *a, *firstbounce, *bouncedata;
-	char *logstr, *bouncelifestr;
+	char *bouncelifestr;
 	struct dirent *dp;
 	struct stat st;
 	pid_t pid, childpid;
@@ -717,8 +696,9 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 		}
 
 		if(childpid > 0) {
-			WRITEMAINTLOG6(5, "UNSUB: ", address, ". Bounced since",
-					bouncedata, ".\n");
+			log_oper(listdir, OPLOGFNAME, "mlmmj-maintd: %s"
+					" unsubscribed due to bouncing since"
+					" %s", address, bouncedata);
 			myfree(address);
 			myfree(bouncedata);
 			do /* Parent waits for the child */
@@ -731,7 +711,7 @@ int unsub_bouncers(const char *listdir, const char *mlmmjunsub)
 		} else {
 			execlp(mlmmjunsub, mlmmjunsub,
 					"-L", listdir,
-					"-a", address, NULL);
+					"-a", address, (char *)NULL);
 			log_error(LOG_ARGS, "Could not execlp %s",
 						mlmmjunsub);
 			return 1;
@@ -799,7 +779,6 @@ int run_digests(const char *listdir, const char *mlmmjsend)
 					digestname);
 			myfree(digestname);
 			myfree(s1);
-			myunlock(fd);
 			close(fd);
 			return 1;
 		}
@@ -815,7 +794,6 @@ int run_digests(const char *listdir, const char *mlmmjsend)
 		myfree(digestname);
 		myfree(indexname);
 		myfree(s1);
-		myunlock(fd);
 		close(fd);
 		return 1;
 	}
@@ -827,7 +805,6 @@ int run_digests(const char *listdir, const char *mlmmjsend)
 		myfree(digestname);
 		myfree(indexname);
 		myfree(s1);
-		myunlock(fd);
 		close(fd);
 		return 1;
 	}
@@ -864,7 +841,6 @@ int run_digests(const char *listdir, const char *mlmmjsend)
 	myfree(indexname);
 	myfree(s1);
 	myfree(s2);
-	myunlock(fd);
 	close(fd);
 	
 	return 0;
