@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003, 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-process.c,v 1.60 2004/11/25 20:00:53 mmj Exp $
+ * $Id: mlmmj-process.c,v 1.66 2005/02/08 18:21:48 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -49,7 +49,8 @@
 #include "prepstdreply.h"
 #include "subscriberfuncs.h"
 #include "memory.h"
-
+#include "log_oper.h"
+#include "chomp.h"
 
 enum action {
 	ALLOW,
@@ -142,7 +143,7 @@ void newmoderated(const char *listdir, const char *mailfilename,
 				"-l", "2",
 				"-L", listdir,
 				"-F", from,
-				"-m", queuefilename, NULL);
+				"-m", queuefilename, (char *)NULL);
 
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
 
@@ -309,9 +310,10 @@ static void print_help(const char *prg)
 
 int main(int argc, char **argv)
 {
-	int i, opt, noprocess = 0, moderated = 0;
+	int i, j, opt, noprocess = 0, moderated = 0;
 	int hdrfd, footfd, rawmailfd, donemailfd;
 	int subonlypost = 0, addrtocc = 1, intocc = 0;
+	int notoccdenymails = 0, noaccessdenymails = 0, nosubonlydenymails = 0;
 	char *listdir = NULL, *mailfile = NULL, *headerfilename = NULL;
 	char *footerfilename = NULL, *donemailname = NULL;
 	char *randomstr = NULL, *mqueuename;
@@ -331,6 +333,7 @@ int main(int argc, char **argv)
 	struct strlist *access_rules = NULL;
 	struct strlist *delheaders = NULL;
 	struct strlist allheaders;
+	struct strlist *alternates = NULL;
 	struct mailhdr readhdrs[] = {
 		{ "From:", 0, NULL },
 		{ "To:", 0, NULL },
@@ -558,12 +561,15 @@ int main(int argc, char **argv)
 			close(rawmailfd);
 			close(donemailfd);
 			unlink(mailfile);
+			log_oper(listdir, OPLOGFNAME, "mlmmj-recieve: sending"
+					" mail from %s to owner",
+					efromemails.emaillist[0]);
 			execlp(mlmmjsend, mlmmjsend,
 					"-l", "4",
 					"-F", efromemails.emaillist[0],
 					"-s", owner,
 					"-a",
-					"-m", donemailname, NULL);
+					"-m", donemailname, (char *)NULL);
 			log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsend);
 			exit(EXIT_FAILURE);
@@ -597,21 +603,39 @@ int main(int argc, char **argv)
 	unlink(mailfile);
 
 	listaddr = getlistaddr(listdir);
+	alternates = ctrlvalues(listdir, "listaddress");
 
 	addrtocc = !(statctrl(listdir, "tocc"));
 	if(addrtocc) {
-		for(i = 0; i < toemails.emailcount; i++)
-			if(strcmp(listaddr, toemails.emaillist[i]) == 0)
-				intocc = 1;
-		for(i = 0; i < ccemails.emailcount; i++)
-			if(strcmp(listaddr, ccemails.emaillist[i]) == 0)
-				intocc = 1;
+		for(i = 0; i < toemails.emailcount; i++) {
+			for(j = 0; j < alternates->count; j++) {
+				chomp(alternates->strs[j]);
+				if(strcmp(alternates->strs[j],
+					toemails.emaillist[i]) == 0)
+					intocc = 1;
+			}
+		}
+		for(i = 0; i < ccemails.emailcount; i++) {
+			for(j = 0; j < alternates->count; j++) {
+				chomp(alternates->strs[j]);
+				if(strcmp(alternates->strs[j],
+					ccemails.emaillist[i]) == 0)
+					intocc = 1;
+			}
+		}
 	}
 
+	for(i = 0; i < alternates->count; i++)
+		myfree(alternates->strs[i]);
+
+	notoccdenymails = statctrl(listdir, "notoccdenymails");
+	
 	if(addrtocc && !intocc) {
 		/* Don't send a mail about denial to the list, but silently
-		 * discard and exit */
-		if (strcasecmp(listaddr, fromemails.emaillist[0]) == 0) {
+		 * discard and exit. Also don't in case of it being turned off
+		 */
+		if ((strcasecmp(listaddr, fromemails.emaillist[0]) == 0) ||
+				notoccdenymails) {
 			myfree(listaddr);
 			unlink(donemailname);
 			myfree(donemailname);
@@ -632,16 +656,18 @@ int main(int argc, char **argv)
 				"-l", "1",
 				"-T", fromemails.emaillist[0],
 				"-F", fromaddr,
-				"-m", queuefilename, NULL);
+				"-m", queuefilename, (char *)NULL);
 
 		log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
 		exit(EXIT_FAILURE);
 	}
 
+	nosubonlydenymails = statctrl(listdir, "nosubonlydenymails");
+
 	subonlypost = statctrl(listdir, "subonlypost");
 	if(subonlypost) {
 		/* Don't send a mail about denial to the list, but silently
-		 * discard and exit */
+		 * discard and exit. */
 		if (strcasecmp(listaddr, fromemails.emaillist[0]) == 0) {
 			myfree(listaddr);
 			unlink(donemailname);
@@ -649,6 +675,12 @@ int main(int argc, char **argv)
 			exit(EXIT_SUCCESS);
 		}
 		if(is_subbed(listdir, fromemails.emaillist[0]) != 0) {
+			if(nosubonlydenymails) {
+				myfree(listaddr);
+				unlink(donemailname);
+				myfree(donemailname);
+				exit(EXIT_SUCCESS);
+			}
 			listname = genlistname(listaddr);
 			listfqdn = genlistfqdn(listaddr);
 			maildata[1] = fromemails.emaillist[0];
@@ -667,26 +699,29 @@ int main(int argc, char **argv)
 					"-l", "1",
 					"-T", fromemails.emaillist[0],
 					"-F", fromaddr,
-					"-m", queuefilename, NULL);
+					"-m", queuefilename, (char *)NULL);
 
 			log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
 			exit(EXIT_FAILURE);
 		}
 	}
 
+	noaccessdenymails = statctrl(listdir, "noaccessdenymails");
+
 	access_rules = ctrlvalues(listdir, "access");
 	if (access_rules) {
 		enum action accret;
 		/* Don't send a mail about denial to the list, but silently
-		 * discard and exit */
-		if (strcasecmp(listaddr, fromemails.emaillist[0]) == 0) {
-			myfree(listaddr);
-			unlink(donemailname);
-			myfree(donemailname);
-			exit(EXIT_SUCCESS);
-		}
+		 * discard and exit. Also do this in case it's turned off */
 		accret = do_access(access_rules, &allheaders);
 		if (accret == DENY) {
+			if ((strcasecmp(listaddr, fromemails.emaillist[0]) ==
+						0) || noaccessdenymails) {
+				myfree(listaddr);
+				unlink(donemailname);
+				myfree(donemailname);
+				exit(EXIT_SUCCESS);
+			}
 			listname = genlistname(listaddr);
 			listfqdn = genlistfqdn(listaddr);
 			fromaddr = concatstr(3, listname, "+bounces-help@",
@@ -706,7 +741,7 @@ int main(int argc, char **argv)
 					"-l", "1",
 					"-T", fromemails.emaillist[0],
 					"-F", fromaddr,
-					"-m", queuefilename, NULL);
+					"-m", queuefilename, (char *)NULL);
 
 			log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsend);
@@ -744,7 +779,7 @@ int main(int argc, char **argv)
 	
 	execlp(mlmmjsend, mlmmjsend,
 				"-L", listdir,
-				"-m", donemailname, NULL);
+				"-m", donemailname, (char *)NULL);
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
 
 	return EXIT_FAILURE;
