@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003, 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-send.c,v 1.66 2004/11/25 15:59:34 mortenp Exp $
+ * $Id: mlmmj-send.c,v 1.76 2005/02/14 22:56:45 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -42,8 +42,8 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include "mlmmj-send.h"
 #include "mlmmj.h"
+#include "mlmmj-send.h"
 #include "mail-functions.h"
 #include "itoa.h"
 #include "incindexfile.h"
@@ -58,15 +58,18 @@
 #include "memory.h"
 #include "statctrl.h"
 #include "ctrlvalue.h"
+#include "mylocking.h"
+#include "getaddrsfromfd.h"
 
 static int addtohdr = 0;
 static int prepmailinmem = 0;
+static int maxverprecips = MAXVERPRECIPS;
 
 char *bounce_from_adr(const char *recipient, const char *listadr,
 		      const char *mailfilename)
 {
 	char *bounceaddr, *myrecipient, *mylistadr;
-	char *indexstr, *listdomain, *a, *mymailfilename;
+	char *indexstr, *listdomain, *a = NULL, *mymailfilename;
 	size_t len;
 
 	mymailfilename = mystrdup(mailfilename);
@@ -87,7 +90,8 @@ char *bounce_from_adr(const char *recipient, const char *listadr,
 		return NULL;
 	}
 	a = strchr(myrecipient, '@');
-	if (a) *a = '=';
+	if (a)
+		*a = '=';
 
 	mylistadr = mystrdup(listadr);
 	if (!mylistadr) {
@@ -115,7 +119,7 @@ char *bounce_from_adr(const char *recipient, const char *listadr,
 		return NULL;
 	}
 	snprintf(bounceaddr, len, "%s%cbounces-%s-%s@%s", mylistadr, RECIPDELIM,
-		 myrecipient, indexstr, listdomain);
+		 indexstr, myrecipient, listdomain);
 
 	myfree(myrecipient);
 	myfree(mylistadr);
@@ -158,8 +162,8 @@ int bouncemail(const char *listdir, const char *mlmmjbounce, const char *from)
 	
 	execlp(mlmmjbounce, mlmmjbounce,
 			"-L", listdir,
-			"-a", addr,
-			"-n", num, NULL);
+			"-a", num,
+			"-n", addr, (char *)NULL);
 
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjbounce);
 
@@ -175,7 +179,7 @@ int send_mail(int sockfd, const char *from, const char *to,
 	int retval = 0;
 	char *reply, *tohdr;
 	
-	retval = write_mail_from(sockfd, from);
+	retval = write_mail_from(sockfd, from, "");
 	if(retval) {
 		log_error(LOG_ARGS, "Could not write MAIL FROM\n");
 		return retval;
@@ -287,13 +291,13 @@ int send_mail(int sockfd, const char *from, const char *to,
 	return 0;
 }
 
-int initsmtp(int *sockfd, const char *relayhost)
+int initsmtp(int *sockfd, const char *relayhost, unsigned short port)
 {
 	int retval = 0;
 	char *reply = NULL;
 	char *myhostname = hostnamestr();
 	
-	init_sockfd(sockfd, relayhost);
+	init_sockfd(sockfd, relayhost, port);
 	
 	if((reply = checkwait_smtpreply(*sockfd, MLMMJ_CONNECT)) != NULL) {
 		log_error(LOG_ARGS, "No proper greeting to our connect"
@@ -317,125 +321,244 @@ int initsmtp(int *sockfd, const char *relayhost)
 int endsmtp(int *sockfd)
 {
 	int retval = 0;
-	char *reply;
+	char *reply = NULL;
+
+	if(*sockfd == -1)
+		return retval;
 	
 	write_quit(*sockfd);
-
-	if((reply = checkwait_smtpreply(*sockfd, MLMMJ_QUIT)) != 0) {
+	reply = checkwait_smtpreply(*sockfd, MLMMJ_QUIT);
+	if(reply) {
+		printf("reply from quit: %s\n", reply);
 		log_error(LOG_ARGS, "Mailserver would not let us QUIT. "
-			  "We close the socket anyway though.");
+			  "We close the socket anyway though. "
+			  "Mailserver reply = [%s]", reply);
 		myfree(reply);
 		retval = MLMMJ_QUIT;
 	}
 
 	close(*sockfd);
 
+	*sockfd = -1;
+
 	return retval;
 }
 
-int send_mail_many(int sockfd, const char *from, const char *replyto,
-		   char *mailmap, size_t mailsize, int subfd,
+int send_mail_verp(int sockfd, struct strlist *addrs, char *mailmap,
+		   size_t mailsize, const char *from, const char *listdir,
+		   const char *hdrs, size_t hdrslen, const char *body,
+		   size_t bodylen, const char *verpextra)
+{
+	int retval, i;
+	char *reply;
+
+	retval = write_mail_from(sockfd, from, verpextra);
+	if(retval) {
+		log_error(LOG_ARGS, "Could not write MAIL FROM\n");
+		return retval;
+	}
+	reply = checkwait_smtpreply(sockfd, MLMMJ_FROM);
+	if(reply) {
+		log_error(LOG_ARGS, "Error in MAIL FROM. Reply = [%s]",
+				reply);
+		myfree(reply);
+		write_rset(sockfd);
+		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		return MLMMJ_FROM;
+	}
+	for(i = 0; i < addrs->count; i++) {
+		retval = write_rcpt_to(sockfd, addrs->strs[i]);
+		if(retval) {
+			log_error(LOG_ARGS, "Could not write RCPT TO:\n");
+			return retval;
+		}
+
+		reply = checkwait_smtpreply(sockfd, MLMMJ_RCPTTO);
+		if(reply) {
+			log_error(LOG_ARGS, "Error in RCPT TO. Reply = [%s]",
+					reply);
+			myfree(reply);
+			return MLMMJ_RCPTTO;
+		}
+	}
+
+	retval = write_data(sockfd);
+	if(retval) {
+		log_error(LOG_ARGS, "Could not write DATA\b");
+		return retval;
+	}
+
+	reply = checkwait_smtpreply(sockfd, MLMMJ_DATA);
+	if(reply) {
+		log_error(LOG_ARGS, "Error with DATA. Reply = [%s]", reply);
+		myfree(reply);
+		write_rset(sockfd);
+		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		return MLMMJ_DATA;
+	}
+
+	if(prepmailinmem) {
+		retval = writen(sockfd, hdrs, hdrslen);
+		if(retval < 0) {
+			log_error(LOG_ARGS, "Could not write mailheaders.\n");
+			return retval;
+		}
+		retval = writen(sockfd, body, bodylen);
+		if(retval < 0) {
+			log_error(LOG_ARGS, "Could not write mailbody.\n");
+			return retval;
+		}
+	} else {
+		retval = write_mailbody_from_map(sockfd, mailmap, mailsize,
+						 NULL);
+		if(retval) {
+			log_error(LOG_ARGS, "Could not write mail\n");
+			return retval;
+		}
+	}
+
+	retval = write_dot(sockfd);
+	if(retval) {
+		log_error(LOG_ARGS, "Could not write <CR><LF>.<CR><LF>\n");
+		return retval;
+	}
+
+	reply = checkwait_smtpreply(sockfd, MLMMJ_DOT);
+	if(reply) {
+		log_error(LOG_ARGS, "Mailserver did not ack end of mail.\n"
+				"<CR><LF>.<CR><LF> was written, to no"
+				"avail. Reply = [%s]", reply);
+		myfree(reply);
+		write_rset(sockfd);
+		checkwait_smtpreply(sockfd, MLMMJ_RSET);
+		return MLMMJ_DOT;
+	}
+
+	return 0;
+}
+
+int send_mail_many_fd(int sockfd, const char *from, const char *replyto,
+		      char *mailmap, size_t mailsize, int subfd,
+		      const char *listaddr, const char *archivefilename,
+		      const char *listdir, const char *mlmmjbounce,
+		      const char *hdrs, size_t hdrslen, const char *body,
+		      size_t bodylen)
+{
+	int res, ret, i;
+	struct strlist stl;
+
+	stl.strs = NULL;
+	stl.count = 0;
+
+	do {
+		res = getaddrsfromfd(&stl, subfd, maxverprecips);
+		if(stl.count == maxverprecips) {
+			ret = send_mail_many_list(sockfd, from, replyto,
+					mailmap, mailsize, &stl, listaddr,
+					archivefilename, listdir, mlmmjbounce,
+					hdrs, hdrslen, body, bodylen);
+			for(i = 0; i < stl.count; i++)
+				myfree(stl.strs[i]);
+			if(ret < 0)
+				return ret;
+			stl.count = 0;
+		}
+	} while(res > 0);
+
+	if(stl.count) {
+		ret = send_mail_many_list(sockfd, from, replyto, mailmap,
+				mailsize, &stl, listaddr, archivefilename,
+				listdir, mlmmjbounce, hdrs, hdrslen, body,
+				bodylen);
+		for(i = 0; i < stl.count; i++)
+			myfree(stl.strs[i]);
+		stl.count = 0;
+		return ret;
+	}
+
+	return 0;
+}
+
+int requeuemail(const char *listdir, const char *index, struct strlist *addrs,
+		int addrcount)
+{
+	int addrfd, i;
+	char *dirname, *addrfilename, *addr;
+	
+	dirname = concatstr(3, listdir, "/requeue/", index);
+	if(mkdir(dirname, 0750) < 0 && errno != EEXIST) {
+		log_error(LOG_ARGS, "Could not mkdir(%s) for "
+				"requeueing. Mail cannot "
+				"be requeued.", dirname);
+		myfree(dirname);
+		return -1;
+	}
+	addrfilename = concatstr(2, dirname, "/subscribers");
+	myfree(dirname);
+	addrfd = open(addrfilename, O_WRONLY|O_CREAT|O_APPEND,
+			S_IRUSR|S_IWUSR);
+	if(addrfd < 0) {
+		log_error(LOG_ARGS, "Could not open %s",
+				addrfilename);
+		myfree(addrfilename);
+		return -1;
+	} else {
+		/* Dump the remaining addresses. We dump the remaining before
+		 * we write the failing address to ensure the potential good
+		 * ones will be tried first when mlmmj-maintd sends out mails
+		 * that have been requeued. addrcount was so far we were */
+		for(i = addrcount + 1; i < addrs->count; i++) {
+			addr = concatstr(2, addrs->strs[i], "\n");
+			if(writen(addrfd, addr, strlen(addr)) < 0) {
+				log_error(LOG_ARGS, "Could not add [%s] "
+						    "to requeue file", addr);
+				return -1;
+			}
+			myfree(addr);
+		}
+		addr = concatstr(2, addrs->strs[addrcount], "\n");
+		if(writen(addrfd, addr, strlen(addr)) < 0) {
+			log_error(LOG_ARGS, "Could not add [%s] to requeue "
+					"file", addr);
+			return -1;
+		}
+		myfree(addr);
+	}
+	myfree(addrfilename);
+	close(addrfd);
+
+	return 0;
+}
+
+int send_mail_many_list(int sockfd, const char *from, const char *replyto,
+		   char *mailmap, size_t mailsize, struct strlist *addrs,
 		   const char *listaddr, const char *archivefilename,
 		   const char *listdir, const char *mlmmjbounce,
 		   const char *hdrs, size_t hdrslen, const char *body,
 		   size_t bodylen)
 {
-	int sendres = 0, addrfd;
-	char *bounceaddr, *addr, *index, *dirname, *addrfilename;
-	char *start, *cur, *next;
-	struct stat st;
-	size_t len;
-
-	if(fstat(subfd, &st) < 0) {
-		log_error(LOG_ARGS, "Could not stat subfd");
-		return -1;
-	}
-
-	start = mmap(0, st.st_size, PROT_READ, MAP_SHARED, subfd, 0);
-	if(start == MAP_FAILED) {
-		log_error(LOG_ARGS, "Could not mmap subfd");
-		return -1;
-	}
-
-	for(next = cur = start; next < start + st.st_size; next++) {
-		if(*next == '\n' || next == start + st.st_size - 1) {
-			len = next - cur;
-			if(next == start + st.st_size - 1 && *next != '\n')
-				len++;
-			addr = mymalloc(len + 1);
-			strncpy(addr, cur, len);
-			addr[len] = '\0';
-			cur = next + 1;
-		} else
-			continue;
+	int res = 0, i;
+	char *bounceaddr, *addr, *index;
 
+	for(i = 0; i < addrs->count; i++) {
+		addr = addrs->strs[i];
 		if(from) {
-			sendres = send_mail(sockfd, from, addr, replyto,
+			res = send_mail(sockfd, from, addr, replyto,
 					    mailmap, mailsize, listdir, NULL,
 					    hdrs, hdrslen, body, bodylen);
 		} else {
 			bounceaddr = bounce_from_adr(addr, listaddr,
 						     archivefilename);
-			sendres = send_mail(sockfd, bounceaddr, addr, replyto,
+			res = send_mail(sockfd, bounceaddr, addr, replyto,
 				  mailmap, mailsize, listdir, mlmmjbounce,
 				  hdrs, hdrslen, body, bodylen);
 			myfree(bounceaddr);
 		}
-		if(sendres && listaddr && archivefilename) {
+		if(res && listaddr && archivefilename) {
 			/* we failed, so save the addresses and bail */
-			index = mybasename(archivefilename);	
-			dirname = concatstr(3, listdir, "/requeue/", index);
-			myfree(index);
-			if(mkdir(dirname, 0750) < 0 && errno != EEXIST) {
-				log_error(LOG_ARGS, "Could not mkdir(%s) for "
-						    "requeueing. Mail cannot "
-						    "be requeued.", dirname);
-				myfree(dirname);
-				myfree(addr);
-				return -1;
-			}
-			addrfilename = concatstr(2, dirname, "/subscribers");
-			myfree(dirname);
-			addrfd = open(addrfilename, O_WRONLY|O_CREAT|O_APPEND,
-							S_IRUSR|S_IWUSR);
-			if(addrfd < 0) {
-				log_error(LOG_ARGS, "Could not open %s",
-						    addrfilename);
-				myfree(addrfilename);
-				myfree(addr);
-				return -1;
-			} else {
-				/* Dump the remaining addresses. We dump the
-				 * remaining before we write the failing
-				 * address to ensure the potential good ones
-				 * will be tried first when mlmmj-maintd
-				 * sends out mails that have been requeued. */
-				if(writen(addrfd, cur, start+st.st_size-cur)
-						< 0) {
-					log_error(LOG_ARGS, "Could not dump "
-							"remaining addresses "
-							"of subfile to "
-							"requeue address "
-							"file");
-				}
-				/* Dirty hack to add newline. */
-				addr[len] = '\n';
-				if(writen(addrfd, addr, len+1) < 0) {
-					addr[len] = '\0';
-					log_error(LOG_ARGS, "Could not add "
-							"[%s] to requeue "
-							"address file", addr);
-				}
-				
-			}
-			
-			myfree(addr);
-			myfree(addrfilename);
-			close(addrfd);
-
-			return -1;
+			index = mybasename(archivefilename);
+			return requeuemail(listdir, index, addrs, i);
 		}
-		myfree(addr);
 	}
 	return 0;
 }	
@@ -470,22 +593,25 @@ static void print_help(const char *prg)
 int main(int argc, char **argv)
 {
 	size_t len = 0, hdrslen, bodylen;
-	int sockfd = 0, mailfd = 0, opt, mindex, subfd = 0, tmpfd;
-	int deletewhensent = 1, sendres, archive = 1, digest = 0;
-	int ctrlarchive;
+	int sockfd = -1, mailfd = 0, opt, mindex = 0, subfd = 0, tmpfd, i;
+	int deletewhensent = 1, sendres = 0, archive = 1, digest = 0;
+	int ctrlarchive, res;
 	char *listaddr = NULL, *mailfilename = NULL, *subfilename = NULL;
 	char *replyto = NULL, *bounceaddr = NULL, *to_addr = NULL;
 	char *relayhost = NULL, *archivefilename = NULL, *tmpstr;
 	char *listctrl = NULL, *subddirname = NULL, *listdir = NULL;
 	char *mlmmjbounce = NULL, *bindir, *mailmap, *probefile, *a;
-	char *body = NULL, *hdrs = NULL, *memmailsizestr = NULL;
-	char relay[16];
+	char *body = NULL, *hdrs = NULL, *memmailsizestr = NULL, *verp = NULL;
+	char relay[16], *listname, *listfqdn, *verpfrom, *maxverprecipsstr;
+	char strindex[32], *reply, *strport;
 	ssize_t memmailsize = 0;
 	DIR *subddir;
 	struct dirent *dp;
 	struct stat st;
 	struct hostent *relayent;
 	uid_t uid;
+	struct strlist stl;
+	unsigned short smtpport = 25;
 
 	CHECKFULLPATH(argv[0]);
 	
@@ -581,6 +707,23 @@ int main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+	verp = ctrlvalue(listdir, "verp");
+	chomp(verp);
+	if(verp == NULL)
+		if(statctrl(listdir, "verp") == 1)
+			verp = mystrdup("");
+
+	maxverprecipsstr = ctrlvalue(listdir, "maxverprecips");
+	if(verp && maxverprecipsstr) {
+		maxverprecips = atol(maxverprecipsstr);
+		myfree(maxverprecipsstr);
+	}
+
+	if(maxverprecips <= 0)
+		maxverprecips = MAXVERPRECIPS;
+
+	stl.strs = NULL;
+	stl.count = 0;
 
 	switch(listctrl[0]) {
 		case '1':
@@ -600,11 +743,17 @@ int main(int argc, char **argv)
 	
 	/* initialize file with mail to send */
 
-	if((mailfd = open(mailfilename, O_RDONLY)) < 0) {
+	if((mailfd = open(mailfilename, O_RDWR)) < 0) {
 	        log_error(LOG_ARGS, "Could not open '%s'", mailfilename);
 		exit(EXIT_FAILURE);
 	}
 
+	if(myexcllock(mailfd) < 0) {
+		log_error(LOG_ARGS, "Could not lock '%s'."
+				"Mail not sent!", mailfilename);
+		exit(EXIT_FAILURE);
+	}
+
 	if(fstat(mailfd, &st) < 0) {
 		log_error(LOG_ARGS, "Could not stat mailfd");
 		exit(EXIT_FAILURE);
@@ -698,6 +847,8 @@ int main(int argc, char **argv)
 			 mindex);
 	}
 
+	itoa(mindex, strindex);
+
 	if(!relayhost) {
 		relayhost = ctrlvalue(listdir, "relayhost");
 		chomp(relayhost);
@@ -715,11 +866,14 @@ int main(int argc, char **argv)
 				strncpy(relay, RELAYHOST, sizeof(relay));
 		}
 	}
+	strport = ctrlvalue(listdir, "smtpport");
+	if(strport)
+		smtpport = (unsigned short)atol(strport);
 
 	switch(listctrl[0]) {
 	case '1': /* A single mail is to be sent */
 	case '6':
-		initsmtp(&sockfd, relay);
+		initsmtp(&sockfd, relay, smtpport);
 		sendres = send_mail(sockfd, bounceaddr, to_addr, replyto,
 				mailmap, st.st_size, listdir, NULL,
 				hdrs, hdrslen, body, bodylen);
@@ -762,17 +916,17 @@ int main(int argc, char **argv)
 		}
 		break;
 	case '2': /* Moderators */
-		initsmtp(&sockfd, relay);
-		if(send_mail_many(sockfd, bounceaddr, NULL, mailmap,
-				  st.st_size, subfd, NULL, NULL, listdir,
-				  NULL, hdrs, hdrslen, body, bodylen))
+		initsmtp(&sockfd, relay, smtpport);
+		if(send_mail_many_fd(sockfd, bounceaddr, NULL, mailmap,
+				     st.st_size, subfd, NULL, NULL, listdir,
+				     NULL, hdrs, hdrslen, body, bodylen))
 			close(sockfd);
 		else
 			endsmtp(&sockfd);
 		break;
 	case '3': /* resending earlier failed mails */
-		initsmtp(&sockfd, relay);
-		if(send_mail_many(sockfd, NULL, NULL, mailmap, st.st_size,
+		initsmtp(&sockfd, relay, smtpport);
+		if(send_mail_many_fd(sockfd, NULL, NULL, mailmap, st.st_size,
 				subfd, listaddr, mailfilename, listdir,
 				mlmmjbounce, hdrs, hdrslen, body, bodylen))
 			close(sockfd);
@@ -781,16 +935,17 @@ int main(int argc, char **argv)
 		unlink(subfilename);
 		break;
 	case '4': /* send mails to owner */
-		initsmtp(&sockfd, relay);
-		if(send_mail_many(sockfd, bounceaddr, NULL, mailmap, st.st_size,
-				subfd, listaddr, mailfilename, listdir,
-				mlmmjbounce, hdrs, hdrslen, body, bodylen))
+		initsmtp(&sockfd, relay, smtpport);
+		if(send_mail_many_fd(sockfd, bounceaddr, NULL, mailmap,
+				st.st_size, subfd, listaddr, mailfilename,
+				listdir, mlmmjbounce, hdrs, hdrslen, body,
+				bodylen))
 			close(sockfd);
 		else
 			endsmtp(&sockfd);
 		break;
 	case '5': /* bounceprobe - handle relayhost local users bouncing*/
-		initsmtp(&sockfd, relay);
+		initsmtp(&sockfd, relay, smtpport);
 		sendres = send_mail(sockfd, bounceaddr, to_addr, replyto,
 				mailmap, st.st_size, listdir, NULL,
 				hdrs, hdrslen, body, bodylen);
@@ -827,6 +982,48 @@ int main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 
+		listname = genlistname(listaddr);	
+		listfqdn = genlistfqdn(listaddr);	
+		verpfrom = concatstr(5, listname, "+bounces-", strindex, "@",
+				listfqdn);
+		myfree(listname);
+		myfree(listfqdn);
+
+		if(digest)
+			verp = NULL;
+
+		if(verp && (strcmp(verp, "postfix") == 0)) {
+			myfree(verp);
+			verp = mystrdup("XVERP=-=");
+		}
+
+		if(addtohdr && verp) {
+			log_error(LOG_ARGS, "Cannot use VERP and add "
+					"To: header. Not sending with "
+					"VERP.");
+			verp = NULL;
+		}
+		
+		if(verp) {
+			initsmtp(&sockfd, relay, smtpport);
+			if(write_mail_from(sockfd, verpfrom, verp)) {
+				log_error(LOG_ARGS,
+						"Could not write MAIL FROM\n");
+				verp = NULL;
+			} else {
+				reply = checkwait_smtpreply(sockfd, MLMMJ_FROM);
+				if(reply) {
+					log_error(LOG_ARGS,
+						"Mailserver did not "
+						"accept verp mail from. "
+						"Not sending with VERP.");
+					myfree(reply);
+					verp = NULL;
+				}
+			}
+			endsmtp(&sockfd);
+		}
+
 		while((dp = readdir(subddir)) != NULL) {
 			if(!strcmp(dp->d_name, "."))
 				continue;
@@ -839,41 +1036,105 @@ int main(int argc, char **argv)
 				myfree(subfilename);
 				continue;
 			}
+			do {
+				res = getaddrsfromfd(&stl, subfd,
+						maxverprecips);
+				if(stl.count == maxverprecips) {
+					initsmtp(&sockfd, relay, smtpport);
+					if(verp) {
+						sendres = send_mail_verp(
+								sockfd, &stl,
+								mailmap,
+								st.st_size,
+								verpfrom,
+								listdir, hdrs,
+								hdrslen, body,
+								bodylen, verp);
+						if(sendres)
+							requeuemail(listdir,
+								strindex,
+								&stl, 0);
+					} else {
+						sendres = send_mail_many_list(
+								sockfd, NULL,
+								NULL, mailmap,
+								st.st_size,
+								&stl,
+								listaddr,
+								archivefilename,
+								listdir,
+								mlmmjbounce,
+								hdrs, hdrslen,
+								body, bodylen);
+					}
+					endsmtp(&sockfd);
+					for(i = 0; i < stl.count; i++)
+						myfree(stl.strs[i]);
+					stl.count = 0;
+				}
+			} while(res > 0);
 			myfree(subfilename);
-
-			initsmtp(&sockfd, relay);
-			sendres = send_mail_many(sockfd, NULL, NULL, mailmap,
-					st.st_size, subfd, listaddr,
-					archivefilename, listdir, mlmmjbounce,
-					hdrs, hdrslen, body, bodylen);
-			if (sendres) {
-				/* If send_mail_many() failed we close the
-				 * connection to the mail server in a brutal
-				 * manner, because we could be in any state
-				 * (DATA for instance). */
-				close(sockfd);
-			} else {
-				endsmtp(&sockfd);
-			}
 			close(subfd);
+
 		}
+		if(stl.count) {
+			initsmtp(&sockfd, relay, smtpport);
+			if(verp) {
+				sendres = send_mail_verp(sockfd, &stl, mailmap,
+						st.st_size, verpfrom, listdir,
+						hdrs, hdrslen, body, bodylen,
+						verp);
+				if(sendres)
+					requeuemail(listdir, strindex, &stl,
+							0);
+			} else {
+				sendres = send_mail_many_list(sockfd, NULL,
+						NULL, mailmap, st.st_size,
+						&stl, listaddr,
+						archivefilename, listdir,
+						mlmmjbounce, hdrs, hdrslen,
+						body, bodylen);
+			}
+			endsmtp(&sockfd);
+			for(i = 0; i < stl.count; i++)
+				myfree(stl.strs[i]);
+			stl.count = 0;
+		}
+
+		if (sendres) {
+			/* If send_mail_many() failed we close the
+			 * connection to the mail server in a brutal
+			 * manner, because we could be in any state
+			 * (DATA for instance). */
+			close(sockfd);
+		} else {
+			endsmtp(&sockfd);
+		}
+		myfree(stl.strs);
+		myfree(verpfrom);
 		closedir(subddir);
 		myfree(subddirname);
 		break;
 	}
 	
+	for(i = 0; i < stl.count; i++)
+		myfree(stl.strs[i]);
+	stl.count = 0;
+
 	myfree(hdrs);
 	myfree(body);
 	myfree(mlmmjbounce);
 	close(sockfd);
 	munmap(mailmap, st.st_size);
 	close(mailfd);
-	
+	myfree(verp);
+
 	if(archive) {
-		if(!ctrlarchive)
+		if(!ctrlarchive) {
 			rename(mailfilename, archivefilename);
-		else
+		} else {
 			unlink(mailfilename);
+		}
 		myfree(archivefilename);
 	} else if(deletewhensent)
 		unlink(mailfilename);
