@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-unsub.c,v 1.31 2004/11/12 01:28:32 mmj Exp $
+ * $Id: mlmmj-unsub.c,v 1.36 2005/02/14 22:56:45 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -82,7 +82,7 @@ void confirm_unsub(const char *listdir, const char *listaddr,
 				"-l", "1",
 				"-T", subaddr,
 				"-F", fromaddr,
-				"-m", queuefilename, NULL);
+				"-m", queuefilename, (char *)NULL);
 
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
 	exit(EXIT_FAILURE);
@@ -130,7 +130,7 @@ void notify_unsub(const char *listdir, const char *listaddr,
 			"-l", "1",
 			"-T", tostr,
 			"-F", fromaddr,
-			"-m", queuefilename, NULL);
+			"-m", queuefilename, (char *)NULL);
 
         log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
         exit(EXIT_FAILURE);
@@ -220,7 +220,7 @@ void generate_unsubconfirm(const char *listdir, const char *listaddr,
 				"-l", "1",
 				"-T", subaddr,
 				"-F", fromaddr,
-				"-m", queuefilename, NULL);
+				"-m", queuefilename, (char *)NULL);
 	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
 	exit(EXIT_FAILURE);
 }
@@ -231,29 +231,34 @@ ssize_t unsubscribe(int subreadfd, int subwritefd, const char *address)
 	struct stat st;
 	char *inmap;
 	size_t len = strlen(address) + 1; /* + 1 for the '\n' */
-	ssize_t writeres, written = 0;
+	ssize_t writeres = 0, written = 0;
 	
 	if(suboff == -1)
-		return 1; /* Did not find subscriber */
+		return -1; /* Did not find subscriber */
 
 	if(fstat(subreadfd, &st) < 0) {
-		log_error(LOG_ARGS, "Could not stat fd");
-		return 1;
+		log_error(LOG_ARGS, "Could not stat subreadfd");
+		return -1;
 	}
 
 	if((inmap = mmap(0, st.st_size, PROT_READ, MAP_SHARED,
 		       subreadfd, 0)) == MAP_FAILED) {
-		log_error(LOG_ARGS, "Could not mmap fd");
-		return 1;
+		log_error(LOG_ARGS, "Could not mmap subreadfd");
+		return -1;
 	}
 
-	if((writeres = writen(subwritefd, inmap, suboff)) < 0)
-		return -1;
+	if(suboff > 0) {
+		writeres = writen(subwritefd, inmap, suboff);
+		if(writeres < 0)
+			return -1;
+	}
 	written += writeres;
 	
-	if((writeres = writen(subwritefd, inmap + suboff + len,
-					st.st_size - len - suboff) < 0))
+	writeres = writen(subwritefd, inmap + len + suboff,
+				st.st_size - len - suboff);
+	if(writeres < 0)
 		return -1;
+
 	written += writeres;
 
 	munmap(inmap, st.st_size);
@@ -283,10 +288,10 @@ int main(int argc, char **argv)
 {
 	int subread, subwrite, rlock, wlock, opt, unsubres, status, nomail = 0;
 	int confirmunsub = 0, unsubconfirm = 0, notifysub = 0, digest = 0;
-	int changeuid = 1, groupwritable = 0;
+	int changeuid = 1, groupwritable = 0, sublock, sublockfd;
 	char *listaddr, *listdir = NULL, *address = NULL, *subreadname = NULL;
 	char *subwritename, *mlmmjsend, *bindir, *subdir;
-	char *subddirname;
+	char *subddirname, *sublockname;
 	off_t suboff;
 	DIR *subddir;
 	struct dirent *dp;
@@ -376,13 +381,13 @@ int main(int argc, char **argv)
 	switch(typesub) {
 		default:
 		case SUB_NORMAL:
-			subdir = mystrdup("/subscribers.d/");
+			subdir = "/subscribers.d/";
 			break;
 		case SUB_DIGEST:
-			subdir = mystrdup("/digesters.d/");
+			subdir = "/digesters.d/";
 			break;
 		case SUB_NOMAIL:
-			subdir = mystrdup("/nomailsubs.d/");
+			subdir = "/nomailsubs.d/";
 			break;
 	}
 		
@@ -397,7 +402,6 @@ int main(int argc, char **argv)
 	if(is_subbed_in(subddirname, address)) {
 		/* Address is not subscribed, so exit silently */
 		myfree(subddirname);
-		myfree(subdir);
 		myfree(listaddr);
 		exit(EXIT_SUCCESS);
 	}
@@ -410,7 +414,6 @@ int main(int argc, char **argv)
 		log_error(LOG_ARGS, "Could not opendir(%s)",
 				    subddirname);
 		myfree(subddirname);
-		myfree(subdir);
 		myfree(listaddr);
 		exit(EXIT_FAILURE);
 	}
@@ -427,7 +430,6 @@ int main(int argc, char **argv)
 
 		subread = open(subreadname, O_RDWR);
 		if(subread == -1) {
-			log_error(LOG_ARGS, "Could not open '%s'", subreadname);
 			myfree(subreadname);
 			continue;
 		}
@@ -439,26 +441,50 @@ int main(int argc, char **argv)
 			continue;
 		}
 
+		/* create a .name.lock file and aquire the lock */
+		sublockname = concatstr(5, listdir, subdir, ".", dp->d_name,
+					".lock");
+		sublockfd = open(sublockname, O_RDWR | O_CREAT,
+						S_IRUSR | S_IWUSR);
+		if(sublockfd < 0) {
+			log_error(LOG_ARGS, "Error opening lock file %s",
+					sublockname);
+			myfree(sublockname);
+			continue;
+		}
+
+		sublock = myexcllock(sublockfd);
+		if(sublock < 0) {
+			log_error(LOG_ARGS, "Error locking '%s' file",
+					sublockname);
+			myfree(sublockname);
+			close(sublockfd);
+			continue;
+		}
+		
 		rlock = myexcllock(subread);
 		if(rlock < 0) {
 			log_error(LOG_ARGS, "Error locking '%s' file",
 					subreadname);
 			close(subread);
+			close(sublockfd);
 			myfree(subreadname);
+			myfree(sublockname);
 			continue;
 		}
 
 		subwritename = concatstr(2, subreadname, ".new");
 
-		subwrite = open(subwritename, O_RDWR | O_CREAT | O_EXCL,
+		subwrite = open(subwritename, O_RDWR | O_CREAT | O_TRUNC,
 				S_IRUSR | S_IWUSR | groupwritable);
 		if(subwrite == -1){
 			log_error(LOG_ARGS, "Could not open '%s'",
 					subwritename);
-			myunlock(subread);
 			close(subread);
+			close(sublockfd);
 			myfree(subreadname);
 			myfree(subwritename);
+			myfree(sublockname);
 			continue;
 		}
 
@@ -466,50 +492,50 @@ int main(int argc, char **argv)
 		if(wlock < 0) {
 			log_error(LOG_ARGS, "Error locking '%s'",
 					subwritename);
-			myunlock(subread);
 			close(subread);
 			close(subwrite);
+			close(sublockfd);
 			myfree(subreadname);
 			myfree(subwritename);
+			myfree(sublockname);
 			continue;
 		}
 
 		unsubres = unsubscribe(subread, subwrite, address);
 		if(unsubres < 0) {
-			myunlock(subread);
-			myunlock(subwrite);
 			close(subread);
 			close(subwrite);
+			close(sublockfd);
 			unlink(subwritename);
 			myfree(subreadname);
 			myfree(subwritename);
+			myfree(sublockname);
 			continue;
 		}
 
-		unlink(subreadname);
-
 		if(unsubres > 0) {
 			if(rename(subwritename, subreadname) < 0) {
 				log_error(LOG_ARGS,
 					"Could not rename '%s' to '%s'",
 					subwritename, subreadname);
-				myunlock(subread);
-				myunlock(subwrite);
 				close(subread);
 				close(subwrite);
 				myfree(subreadname);
 				myfree(subwritename);
 				continue;
 			}
-		} else /* unsubres == 0, no subscribers left */
+		} else { /* unsubres == 0, no subscribers left */
 			unlink(subwritename);
+			unlink(subreadname);
+		}
 
-		myunlock(subread);
-		myunlock(subwrite);
 		close(subread);
 		close(subwrite);
+		close(sublockfd);
 		myfree(subreadname);
 		myfree(subwritename);
+		unlink(sublockname);
+		myfree(sublockname);
 
 		if(confirmunsub) {
 			childpid = fork();
@@ -534,7 +560,6 @@ int main(int argc, char **argv)
         }
 
 	closedir(subddir);
-	myfree(subdir);
 
         notifysub = statctrl(listdir, "notifysub");
 
